package a1_solutionUsingSubclassing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * This class encapsulates a visual mass, which corresponds to an visual element
 * in a 2D image that coheres together and is perceived as a semantic unit
 * 
 * @author mb
 * 
 */
public abstract class VisualMass {

	private final int DOT_DIAMETER = 10;

	private double widthRelativeToSize;
	private double heightRelativeToSize;

	private double centreXPositionRelativeToSize;
	private double centreYPositionRelativeToSize;

	private Color theColour;

	protected VisualMass(double vmCentreXRelativeToImage,
			double vmCentreYRelativeToImage, double vmWidthRelativeToImage,
			double vmHeightRelativeToImage) {
		setWidthRelativeToImage(vmWidthRelativeToImage);
		setHeightRelativeToImage(vmHeightRelativeToImage);
		setCentreXPositionRelativeToImage(vmCentreXRelativeToImage);
		setCentreYPositionRelativeToImage(vmCentreYRelativeToImage);
	}

	public Point getCentreOfVisualMass(Dimension size) {
		int xPos = (int) Math.floor(size.width * centreXPositionRelativeToSize);
		int yPos = (int) Math
				.floor(size.height * centreYPositionRelativeToSize);
		return new Point(xPos, yPos);
	}

	public abstract Shape getOutlineForVisualMass(Dimension size);

	public Shape getCentreDot(Dimension size) {
		Point centre = getCentreOfVisualMass(size);
		Point anchor = new Point(centre.x - DOT_DIAMETER / 2, centre.y
				- DOT_DIAMETER / 2);
		Shape theShape = new Ellipse2D.Float(anchor.x, anchor.y, DOT_DIAMETER,
				DOT_DIAMETER);
		return theShape;
	}

	public double getWidthRelativeToImage() {
		return widthRelativeToSize;
	}

	public void setWidthRelativeToImage(double widthRelativeToImage) {
		this.widthRelativeToSize = widthRelativeToImage;
	}

	public double getHeightRelativeToImage() {
		return heightRelativeToSize;
	}

	public void setHeightRelativeToImage(double heightRelativeToImage) {
		this.heightRelativeToSize = heightRelativeToImage;
	}

	public void setCentreXPositionRelativeToImage(
			double centreXPositionRelativeToImage) {
		this.centreXPositionRelativeToSize = centreXPositionRelativeToImage;
	}

	public double getCentreXPositionRelativeToImage() {
		return centreXPositionRelativeToSize;
	}

	public void setCentreYPositionRelativeToImage(
			double centreYPositionRelativeToImage) {
		this.centreYPositionRelativeToSize = centreYPositionRelativeToImage;
	}

	public double getCentreYPositionRelativeToImage() {
		return centreYPositionRelativeToSize;
	}

	public Color getTheColour() {
		return theColour;
	}

	public void setColour(Color theColour) {
		this.theColour = theColour;
	}

	public String toString() {
		return "\t" + this.getCentreXPositionRelativeToImage() + "\t"
				+ this.getCentreYPositionRelativeToImage() + "\t";
	}

}
